// ============================================================================
//	FPGA_TOP.v	FPGA top module
//	Dedicated to FPGA mag. (CQ pub.)
//
//	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
// ============================================================================
`timescale 1 ns/1 ps

module FPGA_TOP (
	CLK24576, OEn, RCV, VP, VM, VPO, VMO,
	AIC_LRCK, AIC_BCK, AIC_DIN, AIC_I2C_CLK, AIC_SER_DATA,
	LED_Rn, LED_Gn, LED_Bn
);
//----------------------------------------
	input	CLK24576;
	output	OEn, VPO, VMO;
	input	RCV;
	inout	VP, VM;
	output	AIC_LRCK, AIC_BCK, AIC_DIN, AIC_I2C_CLK;
	inout	AIC_SER_DATA;
	output	LED_Rn, LED_Gn, LED_Bn;
//----------------------------------------
	wire	reset48, CLK48, CLK24576, Freq4896, Attach;
	wire	iOEn, iRCV, iVPO, iVMO, BusReset;
	wire	ReqDIR, ReqDEV, ReqIF, ReqEP, ReqType;
	wire [7:0] bRequest;
	wire [15:0] wValue,wIndex,wLength;
	wire [7:0] RxBYTE;
	wire	DataValid, OutDataValid, ERR16, RxEOP, TxEOP;
	wire [1:0] EndPoint;
	wire	RxSOF, RxOUT, RxIN, RxACK, RxDATA0, RxDATA1;
	wire	TxACK, TxNAK, TxDATA, TxDATASQ;
	wire	ReqCanHnd, ReqReject;
	wire[8:0] TxStartAdr, TxAddr;
	wire[7:0] TxBYTE, TxLENGTH;
	wire	Config;
	wire[3:0] ClrFtrOut, ClrFtrIn, EPStatusOut, EPStatusIn;
	wire	SSPND5N, SSPNDN;
	wire [1:0] ALT1;

//------ CODEC ------
	wire [23:0] DAC_DATA;
	wire [8:0] DAC_ADDR;
	wire	DAC_REQ;
	wire	status;

//------ ISO OUT ------
	wire	PlayBack; // 1: Playback / 0: Idle

//--- Currently no used ---
	assign TxACK = 1'b0;
	assign TxNAK = 1'b0;
	assign TxDATA =1'b0;
	assign TxDATASQ= 1'b0;
	assign TxStartAdr=10'b0000000000;
//-------------------------

// USB PHY //
/* Use following lines when you DO NOT use USB PHY chip */
	assign iRCV = VP;
	assign  VP  = (iOEn == 1'b0) ? iVPO : 1'bz;
	assign  VM  = (iOEn == 1'b0) ? iVMO : 1'bz;
/*****************************************/
/* Use following lines when you use USB PHY chip *
	assign iRCV = RCV;
	assign  VP  = iVP;
	assign  VM  = iVM;
/*****************************************/
	assign VPO = iVPO;
	assign VMO = iVMO;
	assign OEn = iOEn;
	assign Attach = 1'b1;

//--- PLL ---
PLL48M PLL_USB(.inclk0(CLK24576), .c0(CLK48));

//--- Reset ---
PORST RST48(.clk(CLK48), .resetn(reset48));
PORST RST24(.clk(CLK24576), .resetn(resetSCK));
//-----------------------
//----- CQ-USB lib -----
//----------------------
CQ_USB CQ (
	.reset(reset48), .CLK48(CLK48), .SCLK(CLK24576), .fs256(Freq4896), .attach(Attach),
	.OEn(iOEn), .RCV(iRCV), .VP(VP), .VM(VM), .VPO(iVPO), .VMO(iVMO), .BusReset(BusReset),
	.ReqDIR(ReqDIR), .ReqDEV(ReqDEV), .ReqIF(ReqIF), .ReqEP(ReqEP), .ReqType(ReqType),
	.bRequest(bRequest), .wValue(wValue), .wIndex(wIndex), .wLength(wLength),
	.RxBYTE(RxBYTE), .DataValid(DataValid), .OutDataValid(OutDataValid),
	.EndPoint(EndPoint), .ERR16(ERR16), .RxEOP(RxEOP), .TxEOP(TxEOP),
	.RxSOF(RxSOF), .RxOUT(RxOUT), .RxIN(RxIN), .RxACK(RxACK), .RxDATA0(RxDATA0), .RxDATA1(RxDATA1),
	.TxACK(TxACK), .TxNAK(TxNAK), .TxDATA(TxDATA), .TxDATASQ(TxDATASQ),
	.ReqCanHnd(ReqCanHnd), .ReqReject(ReqReject),
	.TxStartAdr(TxStartAdr), .TxAddr(TxAddr), .TxBYTE(TxBYTE), .TxLENGTH({2'b00, TxLENGTH}),
	.Config(Config), .ClrFtrOut(ClrFtrOut), .ClrFtrIn(ClrFtrIn), .EPStatusOut(EPStatusOut), .EPStatusIn(EPStatusIn),
	.SSPND5N(SSPND5N), .SSPNDN(SSPNDN)
);

//---------- ISO OUT control-pipe handler --------
C_ISO_OUT ISOC(
	.reset(reset48), .clk(CLK48), .BusReset(BusReset),
	.ReqDEV(ReqDEV), .ReqIF(ReqIF), .ReqEP(ReqEP), . ReqType(ReqType),
	.bRequest(bRequest), .wValue(wValue), .wIndex(wIndex), .wLength(wLength),
	.ByteData(RxBYTE), .DataValid(OutDataValid), .EOP(RxEOP), .Config(Config),
	.ReqCanHnd(ReqCanHnd), .ReqReject(ReqReject),
	.SetFreq(SetFreq), .Freq4896(Freq4896), .ALT1(ALT1),
	.TxAddr(TxAddr[7:0]), .data8(TxBYTE), .MaxLen(TxLENGTH)
);

//---------- ISO OUT endpoint --------
ISO_OUT ISOEP(
	.reset(reset48), .clk(CLK48), .BusReset(BusReset),
	.RxSOF(RxSOF), .RxOUT(RxOUT), .RxDATA0(RxDATA0), .RxDATA1(RxDATA1), .EOP(RxEOP),
	.ERR16(ERR16), .EndPoint(EndPoint), .ISO_OUT_ALT(ALT1),
	.ByteData(RxBYTE), .DataValid(DataValid),
	.PlayBack(PlayBack),
	.DAC_REQ(DAC_REQ), .DAC_ADDR(DAC_ADDR), .DAC_DATA(DAC_DATA)
);

//-- I2S --
I2S IIS(
	.reset(resetSCK), .SCLK(CLK24576), .PlayBack(PlayBack), .fs256(Freq4896),
	.data24(DAC_DATA), .addr(DAC_ADDR), .DAC_REQ(DAC_REQ),
	.LRCK(AIC_LRCK), .BCK(AIC_BCK), .SD(AIC_DIN)
);

//-- I2C (AIC3204) --
I2C IIC(
	.reset(reset48), .clk(CLK48), .init(Config), .status(status),
	.SCL(AIC_I2C_CLK), .SDA(AIC_SER_DATA)
);

//---- LEDs ----
   assign LED_Rn = 1'b1;
   assign LED_Gn = ~PlayBack;
   assign LED_Bn = 1'b1;

endmodule

