-- ============================================================================
--	FPGA_TOP.v	Top module
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- ============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;

entity fpga_top is port(
	CLK24576	: in	std_logic;
----------------------------------
-- USB transceiver I/F
	OEn			: out	std_logic;
	RCV			: in	std_logic;
	VP, VM		: inout	std_logic;
	VPO, VMO	: out	std_logic;
----------------------------------
-- CODEC interface
	AIC_LRCK	: out	std_logic;
	AIC_BCK		: out	std_logic;
	AIC_DIN		: out	std_logic;
--	AIC_DOUT	: in	std_logic;
	AIC_I2C_CLK	: out	std_logic;
	AIC_SER_DATA: inout	std_logic;
-------------
	LED_Rn		: out	std_logic;
	LED_Gn		: out	std_logic;
	LED_Bn		: out	std_logic
);end entity;

architecture RTL of fpga_top is

component PLL48M port(
	inclk0	 : in	std_logic;
	c0		 : out	std_logic
);end component;

component CQ_USB port(
	reset		: in	std_logic;
	CLK48		: in	std_logic;	-- x4 clock (48MHz for full-speed)
	SCLK		: in	std_logic;	-- Audio clock (11.2896MHz/12.288MHz/24.576MHz)
	fs256		: in	std_logic;	-- 1: 256fs / 0: 512fs
------------------------------------------
	attach		: in	std_logic;
-- USB transceiver I/F
	OEn			: out	std_logic;
	RCV			: in	std_logic;
	VP, VM		: in	std_logic;
	VPO, VMO	: out	std_logic;
------------------------------------------
	BusReset	: out	std_logic;
	ReqDIR		: out	std_logic;	-- Request direction
	ReqDEV		: out	std_logic;	-- Request for DEVICE (1: Valid)
	ReqIF		: out	std_logic;	-- Request for I/F (1: Valid)
	ReqEP		: out	std_logic;	-- Request for E/P (1: Valid)
	ReqType		: out	std_logic;	-- Request type (0: std. / 1: class)
	bRequest	: out	std_logic_vector( 7 downto 0); -- bRequest
	wValue		: out	std_logic_vector(15 downto 0);
	wIndex		: out	std_logic_vector(15 downto 0);
	wLength		: out	std_logic_vector(15 downto 0);
	RxBYTE		: out	std_logic_vector( 7 downto 0);
	DataValid	: out	std_logic;	-- RxBYTE can be usable (1: Valid)
	OutDataValid: out	std_logic;	-- RxBYTE can be usable at Control OUT (1: Valid)
	EndPoint	: out	std_logic_vector(1 downto 0);--Supported E/P is 1 to 3
	ERR16		: out	std_logic;	-- CRC16 error
	RxEOP		: out	std_logic;	-- End Of Packet on the bus
	TxEOP		: out	std_logic;	-- End Of Packet (Tx)
	RxSOF		: out	std_logic;	-- Start Of Frame
	RxOUT		: out	std_logic;	-- OUT
	RxIN		: out	std_logic;	-- IN
	RxACK		: out	std_logic;	-- ACK
	RxDATA0,RxDATA1:out	std_logic;	-- DATA0/1

	TxACK, TxNAK: in	std_logic;
	TxDATA		: in	std_logic;
	TxDATASQ	: in	std_logic;		-- Determin DATA0 or DATA1

	ReqCanHnd	: in	std_logic;		-- Request can be handled (1: Can be handled)
	ReqReject	: in	std_logic;		-- Request is rejected (1: Rejected)
	TxStartAdr	: in	std_logic_vector(9 downto 0);
	TxAddr		: out	std_logic_vector(9 downto 0);
	TxBYTE		: in	std_logic_vector(7 downto 0);
	TxLENGTH	: in	std_logic_vector(9 downto 0);
	Config		: out	std_logic;		-- Configuration status (0 Unconfigured, 1: Configured)
	ClrFtrOut	: out	std_logic_vector(3 downto 0);	-- CLEAR_FEATURE was issued for OUT Endpoints (reset to DATA0)
	ClrFtrIn	: out	std_logic_vector(3 downto 0);	-- CLEAR_FEATURE was issued for IN Endpoints (reset to DATA0)
	EPStatusOut	: out	std_logic_vector(3 downto 0);	-- Endpoint status for OUT Endpoints (Bitmap, 1: HALT)
	EPStatusIn	: out	std_logic_vector(3 downto 0);	-- Endpoint status for IN Endpoints (Bitmap, 1: HALT)
------ power management --------
	SSPND5N		: out	std_logic;	-- Pre-suspend signal (5msec), 0:power off (CFG is applied)
	SSPNDN		: out	std_logic	-- AudioPLL power down (0:power off)
);end component;

component C_ISO_OUT port(
	reset, clk	: in	std_logic;
	BusReset	: in	std_logic; -- Bus reset (1: reset)
--	ReqDIR		: in	std_logic; -- Request direction (0: OUT / 1: IN)
	ReqDEV		: in	std_logic; -- Request for DEVICE (1: Valid)
	ReqIF		: in	std_logic; -- Request for I/F (1: Valid)
	ReqEP		: in	std_logic; -- Request for E/P (1: Valid)
	ReqType		: in	std_logic; -- Request type (0: std. / 1: class)
	bRequest	: in	std_logic_vector( 7 downto 0); -- bRequest
	wValue		: in	std_logic_vector(15 downto 0);
	wIndex		: in	std_logic_vector(15 downto 0);
	wLength		: in	std_logic_vector(15 downto 0);
	ByteData	: in	std_logic_vector( 7 downto 0);
	DataValid	: in	std_logic;	-- ByteData was updated (1: Valid)
	EOP			: in	std_logic;	-- End Of Packet
	Config		: in	std_logic;	-- Configuration status (0 Unconfigured, 1: Configured)

	ReqCanHnd	: out	std_logic;	-- Request can be handled
	ReqReject	: out	std_logic;	-- Request is rejected
	Freq4896	: out	std_logic;	-- 0: 48kHz / 1: 96kHz
	ALT1		: out	std_logic_vector(1 downto 0);
	TxAddr		: in	std_logic_vector(7 downto 0);
	data8		: out	std_logic_vector(7 downto 0);
	MaxLen		: out	std_logic_vector(7 downto 0)
);end component;

component ISO_OUT port(
	reset, clk	: in	std_logic;
	BusReset	: in	std_logic;
	RxSOF		: in	std_logic;
	RxOUT		: in	std_logic;
	RxDATA0,RxDATA1:in	std_logic;
	EOP			: in	std_logic;
	EndPoint	: in	std_logic_vector(1 downto 0);
	ISO_OUT_ALT	: in	std_logic_vector(1 downto 0);
	ByteData	: in	std_logic_vector(7 downto 0);
	DataValid	: in	std_logic;
	PlayBack	: out	std_logic;		-- 1: Playback / 0: Idle

	DAC_REQ		: in	std_logic;		-- DAC REQ (1: assreted)
	DAC_ADDR	: in	std_logic_vector( 8 downto 0);
	DAC_DATA	: out	std_logic_vector(23 downto 0)
);end component;

component I2S port(
	reset		: in	std_logic;
	SCLK		: in	std_logic;		-- 24.576MHz (256/512fs)
	PlayBack	: in	std_logic;		-- Play back (1: Enable play back, USB clock domain)
	fs256		: in	std_logic;		-- 0: 512fs (44.1, 48kHz) / 1: 256fs (96kHz) (USB clock domain)
	data24		: in	std_logic_vector(23 downto 0);
	addr		: out	std_logic_vector( 8 downto 0);
	DAC_REQ		: out	std_logic;		-- DAC DATA READ (1: assreted)
	LRCK,BCK,SD	: out	std_logic		-- IIS IF (64BCK)
);end component;

component I2C port(
	reset, clk	: in	std_logic;
	init		: in	std_logic;-- 1: Start init.
	status		: out	std_logic;
	SCL			: out	std_logic;
	SDA			: inout	std_logic );
end component;

component PORST port (
	clk		: in	std_logic;
	resetn	: out	std_logic
);end component;

--- dummy ---
signal	Attach		: std_logic;
---------------------
signal	reset48, resetSCK	: std_logic;
signal	CLK48	: std_logic;
------
signal	iOEn, iRCV	: std_logic;
signal	iVPO, iVMO	: std_logic;
signal	BusReset		: std_logic;
signal	ReqDIR, ReqDEV, ReqIF, ReqEP, ReqType	: std_logic;
signal	bRequest	: std_logic_vector( 7 downto 0); -- bRequest
signal	wValue, wIndex	: std_logic_vector(15 downto 0);
signal	wLength			: std_logic_vector(15 downto 0);
signal	RxBYTE, TxBYTE			: std_logic_vector(7 downto 0);
signal	DataValid, OutDataValid	: std_logic; -- RxData is valid (1: Valid)
signal	TxStartAdr, TxAddr	: std_logic_vector(9 downto 0);
signal	TxLENGTH			: std_logic_vector(7 downto 0);
signal	ReqCanHnd	: std_logic; -- Request can be handled
signal	ReqReject	: std_logic; -- Request is rejected
signal	RxSOF, RxOUT, RxIN, RxACK, RxDATA0, RxDATA1	: std_logic;
signal	TxACK, TxNAK, TxDATA, TxDATASQ				: std_logic;
signal	RxEOP, TxEOP, ERR16	: std_logic;
signal	ALT1		: std_logic_vector(1 downto 0);

signal	EndPoint	: std_logic_vector(1 downto 0);
signal	Config		: std_logic;
signal	ClrFtrOut, ClrFtrIn		: std_logic_vector(3 downto 0);
signal	EPStatusOut, EPStatusIn	: std_logic_vector(3 downto 0);	-- Endpoint status (1: HALT)
signal	SSPND5N			: std_logic;
signal	SSPNDN			: std_logic;

signal	SetFreq		: std_logic;	-- 1: SET_CUR was asserted
signal	Freq4896	: std_logic;	-- 0: 48kHz / 1: 96kHz
------ CODEC ------
signal	DAC_REQ		: std_logic;						-- DAC REQ (1: assreted)
signal	DAC_ADDR	: std_logic_vector( 8 downto 0);
signal	DAC_DATA	: std_logic_vector(23 downto 0);
signal	status		: std_logic;

------ ISO OUT ------
signal	PlayBack	: std_logic;	-- 1: Playback / 0: Idle

begin

--- PLL ---
PLL_USB:PLL48M port map(inclk0 => CLK24576, c0 => CLK48);

--- Reset ---
RST48:PORST port map(CLK48, reset48);
RST24:PORST port map(CLK24576, resetSCK);

Attach	<= '1';
----------------------
----- CQ-USB lib -----
----------------------
CQ: CQ_USB port map(
	reset => reset48, CLK48 => CLK48, SCLK => CLK24576, fs256 => Freq4896, attach => Attach,
	OEn => iOEn, RCV => iRCV, VP => VP, VM => VM, VPO => iVPO, VMO => iVMO, BusReset => BusReset,
	ReqDIR => ReqDIR, ReqDEV => ReqDEV, ReqIF => ReqIF, ReqEP => ReqEP, ReqType => ReqType,
	bRequest => bRequest, wValue => wValue, wIndex => wIndex, wLength => wLength,
	RxBYTE => RxBYTE, DataValid => DataValid, OutDataValid => OutDataValid,
	EndPoint => EndPoint, ERR16 => ERR16, RxEOP => RxEOP, TxEOP => TxEOP,
	RxSOF => RxSOF, RxOUT => RxOUT, RxIN => RxIN, RxACK => RxACK, RxDATA0 => RxDATA0, RxDATA1 => RxDATA1,
	TxACK => TxACK, TxNAK => TxNAK, TxDATA => TxDATA, TxDATASQ => TxDATASQ,
	ReqCanHnd => ReqCanHnd, ReqReject => ReqReject,
	TxStartAdr => TxStartAdr, TxAddr => TxAddr, TxBYTE => TxBYTE, TxLENGTH => "00"&TxLENGTH,
	Config => Config, ClrFtrOut => ClrFtrOut, ClrFtrIn => ClrFtrIn, EPStatusOut => EPStatusOut, EPStatusIn => EPStatusIn,
	SSPND5N => SSPND5N, SSPNDN => SSPNDN
);
OEn <=iOEn;

--- Currently not used ---
TxACK <= '0'; TxNAK <= '0'; TxDATA <='0';
TxDATASQ <= '0';
TxStartAdr<="0000000000";
-------------------------

-- Use following line when you DO NOT use USB PHY chip
iRCV <= VP; VP <= iVPO when(iOEn = '0') else 'Z'; VM <= iVMO when(iOEn = '0') else 'Z';
-- Use following line when you use USB PHY chip
--iRCV <= RCV; VP <= iVP; VM <= iVM;
VPO <= iVPO; VMO <= iVMO;

---------- ISO OUT control-pipe handler --------
ISOC:C_ISO_OUT port map(
	reset48, CLK48, BusReset,ReqDEV,ReqIF,ReqEP,ReqType,bRequest,wValue,wIndex,wLength,
	RxBYTE,OutDataValid, RxEOP, Config,
	ReqCanHnd, ReqReject,
	Freq4896, ALT1,
	TxAddr(7 downto 0), TxBYTE, TxLENGTH
);

---------- ISO OUT endpoint --------
ISOEP:ISO_OUT port map(
	reset48, CLK48, BusReset,
	RxSOF,RxOUT,RxDATA0,RxDATA1, RxEOP, EndPoint, ALT1,
	RxBYTE, DataValid,
	PlayBack,
	DAC_REQ,DAC_ADDR,DAC_DATA
);
--------------------------------

-- I2S --
IIS:I2S port map(resetSCK, CLK24576, PlayBack, Freq4896, DAC_DATA, DAC_ADDR, DAC_REQ, AIC_LRCK, AIC_BCK, AIC_DIN);

-- I2C (AIC3204) --
IIC:I2C port map(reset48, CLK48, Config, status, AIC_I2C_CLK, AIC_SER_DATA);

---- LEDs ----
LED_Rn <= '1';
LED_Gn <= NOT(PlayBack);
LED_Bn <= '1';
end RTL;
