-- ============================================================================
--	I2C_out.vhd	I2C write sub module
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- ============================================================================
library ieee;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity I2C_out is port (
	reset	: in	std_logic; -- reset (active low)
	clk		: in	std_logic; -- Expected freq.=48MHz
	data8	: in	std_logic_vector(7 downto 0); -- 8bits data
	start	: in	std_logic; -- Start I2C (1: start)
	stop	: in	std_logic; -- Stop I2C (1: stop)
	busy	: out	std_logic; -- Busy (1: busy)
	err		: out	std_logic; -- Error (1: Error)
	SCL		: out	std_logic; -- SCL
	SDA		: inout	std_logic  -- SDA
);end entity;

architecture RTL of I2C_out is
	type state_type is (IDLE, START0, START1, DATA, ACK0, STOP0, STOP1);
	signal fsmI2C	: state_type;
	signal buf8		: std_logic_vector(7 downto 0);
	signal div		: std_logic_vector(10 downto 0);
	signal enable_count	: std_logic;
	signal iSDA			: std_logic;
	signal iError		: std_logic;

begin

FSM_I2C: process(reset, clk)
begin
	if(reset = '0') then
		fsmI2C <= IDLE; buf8 <= "00000000"; iError <= '0';
		SCL <= 'Z'; iSDA <= 'Z';
		busy <= '0';
	elsif clk'event and clk='1' then
		case fsmI2C is
			when IDLE =>
				buf8 <= "00000000";
				iError <= '0';
				enable_count <= '0';
				busy <= '0';
				SCL <= 'Z'; iSDA <= 'Z';
				if(start = '1')then
					SCL <= '1'; iSDA <= '1';
					enable_count <= '1';
					busy <= '1';
					fsmI2C <= START0;
				end if;
			when START0 =>
				SCL <= '1'; iSDA <= '1';
				if(div(6 downto 0) = "0111111")then
					fsmI2C <= START1;
				end if;
			when START1 =>
				enable_count <= '1';
				iSDA <= '0';
				 SCL <= div(6);
				if(div(6 downto 0) = "1111110")then
					enable_count <= '0';
				end if;
				if(div(6 downto 0) = "0011111")then
					iSDA <= '0';
					buf8 <= data8;
					fsmI2C <= DATA;
				end if;

			when DATA =>
				enable_count <= '1';
				iSDA <= buf8(7);
				 SCL <= div(6);
				if(div(6 downto 0) = "0011111")then
					buf8 <= buf8(6 downto 0) & '0';
					if(div(10 downto 7) = "1000")then
--						iSDA <= '1';--Explicitly drive '1' to prevent false ACK (due to weak pull-up)
						fsmI2C <= ACK0;
					end if;
				end if;

			when ACK0 =>
				enable_count <= '1';
				busy <= '0';
				iSDA <= 'Z';
				 SCL <= div(6);
				if(div(6 downto 0) = "1111110")then
					enable_count <= '0';
					if(SDA = '1')then
						iError <= '1';
						fsmI2C <= IDLE;
					end if;
				elsif(div(6 downto 0) = "0011111")then
					if(stop = '1')then
						fsmI2C <= STOP0; busy <= '1';
					else
						buf8 <= data8;
						fsmI2C <= DATA; busy <= '1';
						enable_count <= '1';
					end if;
				end if;

			when STOP0 =>
				enable_count <= '1';
				iSDA <= '0';
				if(div(7) = '1')then
					SCL <= '1';
				else
					SCL <= div(6);
				end if;
				if(div(6 downto 0) = "0011111")then
					iSDA <= '1';
					fsmI2C <= STOP1;
				end if;
			when STOP1 =>
				SCL <= '1';
				iSDA <= '1';
				if(div(6 downto 0) = "0011111")then
					fsmI2C <= IDLE;
				end if;
			when others => null;
		end case;
	end if;
end process;
SDA <= iSDA;
err <= iError;

DIVIDER: process(reset, clk)
begin
	if (reset = '0') then
		div <= "00000000000";
	elsif(clk'event and clk='1')then
		if(enable_count = '0')then
			div <= "00000000000";
		else
			div <= div + "00000000001";
		end if;
	end if;
end process;

end RTL;
