-- =============================================================================
--	I2S.vhd
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- =============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.usbpkg.all;

entity I2S is port(
	reset		: in	std_logic;
	SCLK		: in	std_logic;		-- 24.576MHz (256/512fs)
	PlayBack	: in	std_logic;		-- Play back (1: Enable play back, USB clock domain)
	fs256		: in	std_logic;		-- 0: 512fs (44.1, 48kHz) / 1: 256fs (96kHz) (USB clock domain)
	data24		: in	std_logic_vector(23 downto 0);
	addr		: out	std_logic_vector(RAM_AD-1 downto 0);
	DAC_REQ		: out	std_logic;		-- DAC DATA READ (1: assreted [1SCLK] )
	LRCK,BCK,SD	: out	std_logic		-- IIS IF (64BCK)
);end entity;

architecture RTL of I2S is

signal IISCount		: std_logic_vector(8 downto 0);
signal iPbk, jPbk	: std_logic;
signal Play			: std_logic;
signal Freq, iFreq	: std_logic;

signal iAddr	: std_logic_vector(RAM_AD-1 downto 0);
signal Shift24	: std_logic_vector(23 downto 0);

begin

SYN:process(reset, SCLK)
begin
	if(reset = '0')then
		iFreq<= '0'; Freq<= '0';
		iPbk <= '0'; jPbk <= '0';
		Play <= '0';
	elsif(SCLK'event AND SCLK = '1')then
		iFreq <= fs256; Freq <= iFreq;
		iPbk <= PlayBack; jPbk <= iPbk;
		if(	  (Play = '0') AND ((IISCount(8 downto 1) = "11111111") AND (jpbk = '1')) )then
			Play <= '1';
		elsif((Play = '1') AND ((IISCount(8 downto 1) = "11111111") AND (jpbk = '0')) )then
			Play <= '0';
		end if;
	end if;
end process;

CNTR:process(reset, SCLK)
begin
	if(reset = '0')then
		IISCount	<= X"64"&'0';
	elsif(SCLK'event AND SCLK = '1')then
		if(Freq = '0')then -- 512fs
			IISCount	<= IISCount + X"01";
		else
			IISCount	<= IISCount + X"02";
		end if;
	end if;
end process;

DAC_REQ	<= IISCount(7);

SFT:process(reset, SCLK)
begin
	if(reset = '0')then
		Shift24	<= X"000000";
		iAddr	<= CONV_STD_LOGIC_VECTOR(0,  RAM_AD);
	elsif(SCLK'event AND SCLK = '1')then
		if(Play = '0')then
			Shift24	<= X"000000";
			iAddr	<= CONV_STD_LOGIC_VECTOR(0,  RAM_AD);
		else -- Đ
			if(Freq = '0')then -- 512fs
				if(IISCount(7 downto 0) = X"07")then	-- Data ready
					Shift24 <= data24;
					iAddr	<= iAddr + CONV_STD_LOGIC_VECTOR(1,  RAM_AD);
				else
					if(IISCount(2 downto 0) = "111")then
						Shift24 <= Shift24(22 downto 0) & '0';
					end if;
				end if;
			else -- 256fs
				if(IISCount(7 downto 1) = "0000011")then	-- Data ready
					Shift24 <= data24;
					iAddr	<= iAddr + CONV_STD_LOGIC_VECTOR(1,  RAM_AD);
				else
					if(IISCount(2 downto 1) = "11")then
						Shift24 <= Shift24(22 downto 0) & '0';
					end if;
				end if;
			end if;
		end if;
	end if;
end process;

addr<=iAddr;
LRCK<= IISCount(8);
BCK	<= IISCount(2);
SD	<= Shift24(23);

end RTL;
