-- =============================================================================
-- ISO_OUT.vhd	ISO out endpoint
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- =============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.usbpkg.all;

entity ISO_OUT is port(
	reset, clk	: in	std_logic;
	BusReset	: in	std_logic;
	RxSOF		: in	std_logic;
	RxOUT		: in	std_logic;
	RxDATA0,RxDATA1:in	std_logic;
	EOP			: in	std_logic;
--	ERR16		: in	std_logic;
	EndPoint	: in	std_logic_vector(1 downto 0);
	ISO_OUT_ALT	: in	std_logic_vector(1 downto 0);
	ByteData	: in	std_logic_vector(7 downto 0);
	DataValid	: in	std_logic;
	PlayBack	: out	std_logic;		-- 1: Playback / 0: Idle

	DAC_REQ		: in	std_logic;		-- DAC REQ (1: assreted)
	DAC_ADDR	: in	std_logic_vector(RAM_AD-1 downto 0);
	DAC_DATA	: out	std_logic_vector(23 downto 0)
);end entity;

architecture RTL of ISO_OUT is

component iRAM port(
	reset, clk48	: in	std_logic;
	USB_REQ			: in	std_logic;	-- USB REQ (1: assreted)
	USB_ADDR		: in	std_logic_vector(RAM_AD-1 downto 0);
	USB_DATA		: in	std_logic_vector(23 downto 0);
	DAC_REQ			: in	std_logic;	-- DAC REQ (1: assreted)
	DAC_ADDR		: in	std_logic_vector(RAM_AD-1 downto 0);
	DAC_DATA		: out	std_logic_vector(23 downto 0)
);end component;

type	typePbk is (IDLE, RxDATA, SOF, GO, Playing, Stopping);
type	typeSeq is (IDLE, RxDATA, DATA0, CRC16);
signal	fsmPBK		: typePbk;
signal	fsmOUT		: typeSeq;
signal	iDataSeq	: std_logic;
signal	iwr			: std_logic;
signal	iPlayBack	: std_logic;
signal	iStop		: std_logic;
signal	data24		: std_logic_vector(23 downto 0);
signal	SampleCntr	: std_logic_vector( 2 downto 0);
signal	iAddrWr		: std_logic_vector(RAM_AD-1 downto 0);
signal	iSOF		: std_logic;

begin
process(reset, clk)
begin
if(reset = '0')then
	fsmOUT	<= IDLE;
	data24	<= X"000000";
	SampleCntr<= "000";
	iwr		<= '0';
elsif(clk'event AND clk = '1')then
	iwr		<= '0';
	if(		(ISO_OUT_ALT= "00")		--> Zero B/W
		OR	(BusReset	= '0') )then
		SampleCntr<= "000";
		fsmOUT <= IDLE;
	else
		case fsmOUT is
			when IDLE =>
				SampleCntr<= "000";
				if( (RxOUT = '1') AND (EndPoint = ISO_OUT_EP(1 downto 0)) )then --> OUT token
					fsmOUT <= RxDATA;
				end if;
			when RxDATA =>
				if(RxDATA0 = '1')then
					if(iDataSeq = '0')then
						fsmOUT <= DATA0;
					else
--						fsmOUT <= IDLE;	-- False sequence
						fsmOUT <= DATA0;
					end if;
				elsif(RxDATA1 = '1')then
					if(iDataSeq = '1')then
						fsmOUT <= DATA0;
					else
--						fsmOUT <= IDLE;	-- False sequence
						fsmOUT <= DATA0;
					end if;
				end if;
			when DATA0 =>
				iwr		<= '0';
				if(EOP = '1')then
					fsmOUT <= IDLE;
				elsif(DataValid = '1')then
					SampleCntr<= SampleCntr + "001";
					data24	<= ByteData & data24(23 downto 8);
						if(SampleCntr = "010")then		-- Lch
							iwr		<= '1';
						elsif(SampleCntr = "101")then	-- Rch
							SampleCntr<= "000";
							iwr		<= '1';
						end if;
				end if;
			when others => fsmOUT <= IDLE;
		end case;
	end if;
end if;
end process;

SQ:process(reset, clk)
begin
if(reset = '0')then
	iDataSeq<= '0'; iSOF<='0';
elsif(clk'event AND clk = '1')then
	iSOF <= RxSOF;
	if((ISO_OUT_ALT= "00")OR(BusReset	= '0'))then
		iDataSeq<= '0';
	elsif((RxSOF = '1') AND (iSOF = '0'))then
		iDataSeq <= NOT(iDataSeq);
	end if;
end if;
end process;

process(reset, clk)
begin
if(reset = '0')then
	iAddrWr	<= CONV_STD_LOGIC_VECTOR(0, RAM_AD);
elsif(clk'event AND clk = '1')then
	if(		(ISO_OUT_ALT= "00")		--> Zero B/W
		OR	(BusReset	= '0')
		OR	(iStop	= '1') )then
		iAddrWr	<= CONV_STD_LOGIC_VECTOR(0, RAM_AD);
	elsif(iwr = '1')then
		iAddrWr	<=	  iAddrWr + CONV_STD_LOGIC_VECTOR(1, RAM_AD);
	elsif(RxSOF = '1')then
		iAddrWr	<= ( iAddrWr AND CONV_STD_LOGIC_VECTOR(-2, RAM_AD) );
	end if;
end if;
end process;


RM:iRAM port map(reset, clk, iwr, iAddrWr, data24, DAC_REQ, DAC_ADDR, DAC_DATA);


------ Play back signal --------
process(reset, clk)
begin
if(reset = '0')then
	fsmPBK	<= IDLE;
	iPlayBack<= '0'; iStop <= '0';
elsif(clk'event AND clk = '1')then
	iStop <= '0';
	if(		(ISO_OUT_ALT= "00")		--> Zero B/W
		OR	(BusReset	= '0') )then
		fsmPBK <= IDLE;
		iPlayBack<= '0';
	else
		case fsmPBK is
			when IDLE =>
				iPlayBack<= '0';
				if(fsmOUT = RxDATA)then
					fsmPBK <= RxDATA;
				end if;
			when RxDATA =>
				iPlayBack<= '0';
				if(RxSOF = '1')then
					iPlayBack<= '1';
					fsmPBK <= GO;
				end if;
			when GO =>
				iPlayBack<= '1';
				if(DAC_REQ = '1')then
					if(iAddrWr /= DAC_ADDR)then
						fsmPBK <= Playing;
					end if;
				end if;
			when Playing =>
				iPlayBack<= '1';
				if(DAC_REQ = '1')then
					if(iAddrWr = DAC_ADDR)then
						iPlayBack<='0';
						iStop	<= '1';
						fsmPBK	<= Stopping;
					end if;
				end if;
			when Stopping =>
				iPlayBack<='0';
				iStop	<= '1';-
				if(RxSOF = '1')then
					fsmPBK	<= IDLE;
				end if;
			when others => fsmPBK <= IDLE;
		end case;
	end if;
end if;
end process;
PlayBack<= iPlayBack;
end RTL;
