-- =============================================================================
--	iRAM.vhd	Shared RAM arbitrator
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- =============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all;
--use IEEE.std_logic_unsigned.all;
use work.usbpkg.all;

entity iRAM is port(
	reset, clk48	: in	std_logic;
	USB_REQ			: in	std_logic;					-- USB REQ (1: assreted)
	USB_ADDR		: in	std_logic_vector(RAM_AD-1 downto 0);
	USB_DATA		: in	std_logic_vector(23 downto 0);
	DAC_REQ			: in	std_logic;					-- DAC REQ (1: assreted)
	DAC_ADDR		: in	std_logic_vector(RAM_AD-1 downto 0);
	DAC_DATA		: out	std_logic_vector(23 downto 0)
);end entity;

architecture RTL of iRAM is
component RAM24 port(
	address	: in	std_logic_vector(RAM_AD-1 downto 0);
	clock	: in	std_logic;
	data	: in	std_logic_vector(23 downto 0);
	wren	: in	std_logic;
	q		: out	std_logic_vector(23 downto 0)
);end component;

signal DAC_REQ0, DAC_REQ48		: std_logic;
signal we		: std_logic;	-- 1: Write
signal rd0, rd1	: std_logic;	-- 1: Read
signal address	: std_logic_vector(RAM_AD-1 downto 0);
signal dout: std_logic_vector(23 downto 0);

begin

SYNC:process(reset, clk48)
begin
	if(reset = '0')then
		DAC_REQ0	<= '0';
		DAC_REQ48	<= '0';
	elsif(clk48'event AND clk48 = '1')then
		DAC_REQ0	<= DAC_REQ;
		DAC_REQ48	<= DAC_REQ0;
	end if;
end process;

ADR:process(reset, clk48)
begin
	if(reset = '0')then
		address	<= CONV_STD_LOGIC_VECTOR(0,  RAM_AD);
		we		<= '0';
		rd0		<= '0';
	elsif(clk48'event AND clk48 = '1')then
		address	<= CONV_STD_LOGIC_VECTOR(0,  RAM_AD);
		we		<= '0';
		rd0		<= '0';
		if(USB_REQ = '1')then
			address <= USB_ADDR;
			we		<= '1';
		elsif(DAC_REQ48 = '1')then
			address <= DAC_ADDR;
			rd0		<= '1';
		end if;
	end if;
end process;

DAT:process(reset, clk48)
begin
	if(reset = '0')then
		DAC_DATA	<= X"000000";
		rd1	<= '0';
	elsif(clk48'event AND clk48 = '1')then
		rd1	<= rd0;		-- DATA will be ready at next clock
		if(rd1 = '1')then
			DAC_DATA <= dout;
		end if;
	end if;
end process;

R24:RAM24 port map(address => address, clock => clk48, data => USB_DATA, wren => we, q => dout);
end RTL;
